//********************************************************************
UNIT Administrator;
//********************************************************************
INTERFACE

USES OSSupport,GraphicSupport,GUI;
//********************************************************************
TYPE

//********************************************************************
   TAdministrator = Class(TGUIItem)
   Private
      fDone: Boolean;
      fWindowClassName: String;
      fIdleWindow: TGUIWindow;

      Function GetWindowByHandle(AHandle: Integer): TGUIWindow;

      // Methods that are called automatically by eventpipeline
      Procedure Idle; Reintroduce;
      Procedure Paint(AReceiver: LongWord); Reintroduce;
      Procedure Move(AReceiver: LongWord; X,Y: Integer); Reintroduce;
      Procedure Resize(AReceiver: LongWord; X,Y: Integer); Reintroduce;

      Procedure KeyDown(AReceiver: LongWord; AKey,PreviousKey: LongWord); Reintroduce;
      Procedure KeyUp(AReceiver: LongWord; AKey,PreviousKey: LongWord); Reintroduce;
      Procedure MouseBtnLDown(AReceiver: LongWord; X,Y,AddKeys: Integer); Reintroduce;
      Procedure MouseBtnLUp(AReceiver: LongWord; X,Y,AddKeys: Integer); Reintroduce;
      Procedure MouseBtnRDown(AReceiver: LongWord; X,Y,AddKeys: Integer); Reintroduce;
      Procedure MouseBtnMDown(AReceiver: LongWord; X,Y,AddKeys: Integer); Reintroduce;
      Procedure MouseMove(AReceiver: LongWord; X,Y: Integer); Reintroduce;
      Procedure MouseWheelDown(AReceiver: LongWord; X,Y,ADelta: Integer); Reintroduce;
      Procedure MouseWheelUp(AReceiver: LongWord; X,Y,ADelta: Integer); Reintroduce;
   Public
      Property Done: Boolean Read fDone;
      Property IdleWindow: TGUIWindow Read fIdleWindow Write fIdleWindow;
      Property ItemByName[Index: String]: TGUIItem Read TREEItemByName;

      Constructor Create(AWindowClassName: String); Reintroduce;
      Destructor Destroy; Override;
      Procedure InitEventPipeline;
      Procedure ProcessEvent;
      Procedure DestroyWindow(AWindow: TGUIWindow);
   End;
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
//   T A D M I N I S T R A T O R
//********************************************************************
Constructor TAdministrator.Create(AWindowClassName: String);
Begin
   Inherited Create('GUIAdmin',Nil,0,0,0,0);

   If Not InitializeGraphLib
      Then Begin
         fDone:= True;
      End
      Else Begin
         fDone:= False;
         fWindowClassName:= AWindowClassName;
         WindowClassCreate(AWindowClassName);
      End;
End;
//********************************************************************
Destructor TAdministrator.Destroy;
Begin
   // Inform Operating System
   WindowClassDelete(fWindowClassName);
   EventTerminateApplication;

   Inherited Destroy;
End;
//********************************************************************
Procedure TAdministrator.InitEventPipeline;
Begin
   EventPipeline.OnMove          := Move;
   EventPipeline.OnPaint         := Paint;
   EventPipeline.OnResize        := Resize;
   EventPipeline.OnKeyDown       := KeyDown;
   EventPipeline.OnKeyUp         := KeyUp;
   EventPipeline.OnMouseBtnLDown := MouseBtnLDown;
   EventPipeline.OnMouseBtnLUp   := MouseBtnLUp;
   EventPipeline.OnMouseBtnRDown := MouseBtnRDown;
   EventPipeline.OnMouseBtnMDown := MouseBtnMDown;
   EventPipeline.OnMouseMove     := MouseMove;
   EventPipeline.OnMouseWheelDown:= MouseWheelDown;
   EventPipeline.OnMouseWheelUp  := MouseWheelUp;
End;
//********************************************************************
Procedure TAdministrator.ProcessEvent;
Begin
   If Not EventDispatch
      Then Idle;
End;
//********************************************************************
Function TAdministrator.GetWindowByHandle(AHandle: Integer): TGUIWindow;
Var LocalWindow,FoundWindow: TGUIWindow; I: Integer;
Begin
   FoundWindow:= Nil;

   With fChildren Do If Count > 0 Then Begin
      I:= 0;
      While (I < Count) And (FoundWindow = Nil) Do Begin
         LocalWindow:= List[I];
         If LocalWindow.Handle = AHandle
            Then FoundWindow:= LocalWindow
            Else Inc(I); //=================>
      End;
   End;
   Result:= FoundWindow;
End;
//********************************************************************
//   T A D M I N I S T R A T O R  -  Methods that are called
//                                   automatically by eventpipeline
//********************************************************************
Procedure TAdministrator.DestroyWindow(AWindow: TGUIWindow);
Var LocalWindow: TGUIWindow;
Begin
   If AWindow <> Nil
      Then Begin
         LocalWindow:= GetWindowByHandle(AWindow.Handle);

         If LocalWindow <> Nil Then Begin

         If LocalWindow.IsMainWindow
            Then fDone:= True
            Else Begin
               // Unregisters and frees the window
               UnRegister(LocalWindow);
               LocalWindow.Free;
            End;
            
         End;
      End;
End;
//********************************************************************
Procedure TAdministrator.Idle;
Begin
   If IdleWindow <> Nil
      Then IdleWindow.Idle;
End;
//********************************************************************
Procedure TAdministrator.Move(AReceiver: LongWord; X,Y: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.Move(X,Y);
End;
//********************************************************************
Procedure TAdministrator.Paint(AReceiver: LongWord);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.TREEPaint;
End;
//********************************************************************
Procedure TAdministrator.Resize(AReceiver: LongWord; X,Y: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.Resize(X,Y);
End;
//********************************************************************
Procedure TAdministrator.KeyDown(AReceiver: LongWord;
                        AKey,PreviousKey: LongWord);
Var LocalWindow: TGUIWindow; 
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.KeyDown(AKey,PreviousKey);
   Inherited KeyDown(AKey,PreviousKey);
End;
//********************************************************************
Procedure TAdministrator.KeyUp(AReceiver: LongWord;
                      AKey,PreviousKey: LongWord);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.KeyUp(AKey,PreviousKey);
   Inherited KeyUp(AKey,PreviousKey);
End;
//********************************************************************
// Get focus in Mouseklick
Procedure TAdministrator.MouseBtnLDown(AReceiver: LongWord;
                                     X,Y,AddKeys: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseBtnLDown(X,Y,AddKeys);
   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TAdministrator.MouseBtnLUp(AReceiver: LongWord;
                                   X,Y,AddKeys: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseBtnLUp(X,Y,AddKeys);
   Inherited MouseBtnLUp(X,Y,AddKeys);
End;
//********************************************************************
Procedure TAdministrator.MouseBtnRDown(AReceiver: LongWord;
                                     X,Y,AddKeys: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseBtnRDown(X,Y,AddKeys);
   Inherited MouseBtnRDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TAdministrator.MouseBtnMDown(AReceiver: LongWord;
                                     X,Y,AddKeys: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseBtnMDown(X,Y,AddKeys);
   Inherited MouseBtnMDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TAdministrator.MouseMove(AReceiver: LongWord; X,Y: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseMove(X,Y);
   Inherited MouseMove(X,Y);
End;
//********************************************************************
Procedure TAdministrator.MouseWheelDown(AReceiver: LongWord;
                                    X,Y,ADelta: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseWheelDown(X,Y,ADelta);
   Inherited MouseWheelDown(X,Y,ADelta);
End;
//********************************************************************
Procedure TAdministrator.MouseWheelUp(AReceiver: LongWord;
                                  X,Y,ADelta: Integer);
Var LocalWindow: TGUIWindow;
Begin
   LocalWindow:= GetWindowByHandle(AReceiver);
   If LocalWindow <> Nil
      Then LocalWindow.MouseWheelUp(X,Y,ADelta);
   Inherited MouseWheelUp(X,Y,ADelta);
End;
//********************************************************************
//********************************************************************
End.
