//********************************************************************
UNIT GUIDescendants;
//********************************************************************
INTERFACE

USES SysUtils, Types, Classes, GUI, GraphicSupport, OSSupport;
//********************************************************************
TYPE
//********************************************************************
   TGUIButton = Class(TGUIItem)
   Protected
      Procedure Paint; Override;
   Public
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer;
         ACaption: String); Reintroduce;

      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
      Procedure MouseBtnLUp(X,Y,AddKeys: Integer); Override;
      Procedure MouseMove(X,Y: Integer); Override;
   End;
//********************************************************************
   TGUISymbolBtn = Class(TGUIButton)
   Private
      fSpecification: Integer;
      fSymbol: TSymbol;
   Protected
      Procedure Paint; Override;
   Public
      Property Specification: Integer
         Read fSpecification Write fSpecification;
      Property Symbol: TSymbol Read fSymbol Write fSymbol;
   End;
//********************************************************************
   TMovement = (Horizontal,Vertical);
   TGUISlider = Class(TGUIButton)
   Private
      fBorder1,fBorder2: Integer;
      fMovement: TMovement;
      StartX,StartY: Integer;
   Protected
      Procedure Move(X,Y: Integer); Override;
   Public
      Property Border1: Integer Read fBorder1 Write fBorder1;
      Property Border2: Integer Read fBorder2 Write fBorder2;

      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer;
         AMovement: TMovement); Reintroduce;
      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
      Procedure MouseBtnLUp(X,Y,AddKeys: Integer); Override;
      Procedure MouseMove(X,Y: Integer); Override;
   End;
//********************************************************************
   TGUIPanel = Class(TGUIItem)
   Private
      //fDragMode: Byte;
      StartX,StartY: Integer;
   Protected
      Procedure Paint; Override;
   Public
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer); Reintroduce;

      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
      Procedure MouseBtnLUp(X,Y,AddKeys: Integer); Override;
      Procedure MouseMove(X,Y: Integer); Override;
      Procedure MouseWheelDown(X,Y,ADelta: Integer); Override;
      Procedure MouseWheelUp(X,Y,ADelta: Integer); Override;
   End;
//********************************************************************
   TGUILabel = Class(TGUIItem)
   Protected
      Procedure Paint; Override;
   Public
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer;
         ACaption: String); Reintroduce;
   End;
//********************************************************************
   TGUICheckBox = Class(TGUIItem)
   Private
      fBoxSize: Integer;
      fBoxColor: TRGBAColor;
      fChecked: Boolean;
      fHook: TSymbol;
      fLabel: TGUILabel;
      fMargin: Integer;
   Protected
      Procedure Paint; Override;
      Procedure SetAlign(Align: TAlignment); Override;
   Public
      Property Checked: Boolean Read fChecked Write fChecked;
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer;
         ACaption: String); Reintroduce;
      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
   End;
//********************************************************************
   TGUIImage = Class(TGUIItem)
   End;
//********************************************************************
   TScrollCode = (scLineUp,scLineDown,scPageUp,scPageDown,scPosition);
   TScrollEvent = Procedure (AScrollCode: TScrollCode;
                    AScrollPos: Integer) Of Object;
//********************************************************************
   TGUIScrollBar = Class(TGUIItem)
   Private
      fFirstBtn,fSecondBtn: TGUISymbolBtn;
      fSlider: TGUISlider;
      fPosition,fMax,fMin,fLargeChange,fSmallChange: Integer;

      fOnScroll: TScrollEvent;

      Procedure CreateButtons; Virtual; Abstract;
      Procedure Decrease(X,Y,AddKeys: Integer);
      Procedure Increase(X,Y,AddKeys: Integer);
      Procedure SetAll; Virtual; Abstract;
      Procedure SetSlider; Virtual; Abstract;

      Procedure CalcPosition; Virtual; Abstract;
      Procedure SetPosition(APosition: Integer);
      Procedure SetMax(AMax: Integer);
      Procedure SetMin(AMin: Integer);
      Procedure SetLargeChange(ALargeChange: Integer);
      Procedure SetSmallChange(ASmallChange: Integer);
   Protected
      Procedure Paint; Override;
      Procedure Move(X,Y: Integer); Override;
      Procedure SetAlign(Align: TAlignment); Override;
   Public
      Property Position: Integer Read fPosition Write SetPosition;
      Property Max: Integer Read fMax Write SetMax;
      Property Min: Integer Read fMin Write SetMin;
      Property LargeChange: Integer Read fLargeChange Write SetLargeChange;
      Property SmallChange: Integer Read fSmallChange Write SetSmallChange;
      Property OnScroll: TScrollEvent Read fOnScroll Write fOnScroll;

      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer); Reintroduce;

      Procedure MouseMove(X,Y: Integer); Override;
      Procedure Scroll(AScrollCode: TScrollCode; AScrollPos: Integer);
      Procedure SliderShift(X,Y: Integer);
   End;
//********************************************************************
   TGUIScrollHorizBar = Class(TGUIScrollBar)
   Private
      Procedure CalcPosition; Override;
      Procedure CreateButtons; Override;
      Procedure SetAll; Override;
      Procedure SetSlider; Override;
   Public
      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
   End;
//********************************************************************
   TGUIScrollVertBar = Class(TGUIScrollBar)
   Private
      Procedure CalcPosition; Override;
      Procedure CreateButtons; Override;
      Procedure SetAll; Override;
      Procedure SetSlider; Override;
   Public
      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
   End;
//********************************************************************
   TGUIProgressBar = Class(TGUIItem)
   Private
      fColor: TRGBAColor;
      fValue: Single;
      Procedure SetValue(Value: Single);
   Protected
      Procedure Paint; Override;
   Public
      Property Value: Single Read fValue Write SetValue;
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer); Reintroduce;
   End;
//********************************************************************
   TGUIListBox = Class(TGUIItem)
   Private
      fItems: TStringList;
      fItemIndex: Integer;
      fTopItem: Integer;
      fLineHeight: Integer;
      fLinesPerPage: Integer;
      fMargin: Integer;
      fScrollBar: TGUIScrollVertBar;  // *****
      Function GetItem(AIndex: Integer): String;
      Procedure SetItem(AIndex: Integer; AString: String);
      Procedure ShowPage;
      Procedure TrackPosition(AScrollCode: TScrollCode;
                    AScrollPos: Integer);
      Function ValidIndex(AIndex: Integer): Boolean;
   Protected
      Procedure Paint; Override;
   Public
      Property Items[Index: Integer]: String Read GetItem Write SetItem;
      Property ItemIndex: Integer Read fItemIndex Write fItemIndex ;
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer); Reintroduce;
      Destructor Destroy; Override;
      Procedure MouseBtnLDown(X,Y,AddKeys: Integer); Override;
   End;
//********************************************************************
   TGUIComboBox = Class(TGUIItem)
   Private
      fItems: TStringList;
      fItemIndex: Integer;
      fScrollBar: TGUIScrollVertBar;  // *****
   Protected
      Procedure Paint; Override;
   Public
      Property Items: TStringList Read fItems;
      Property ItemIndex: Integer Read fItemIndex;
      Constructor Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer); Reintroduce;
      Destructor Destroy; Override;
   End;
//********************************************************************
   //TGUIRadioBox = Class(TGUIItem) ????
   //TGUIEdit = Class(TGUIItem)
   //TGUIListView = Class(TGUIItem)
   //TGUITreeView = Class(TGUIItem)
   //TGUIMemo = Class(TGUIItem)
   //TGUITabSheet = Class(TGUIItem)
   //TGUIDropDownMenu = Class(TGUIItem)
   //TGUIStringGrid = Class(TGUIItem)
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
//   T G U I B U T T O N
//********************************************************************
Constructor TGUIButton.Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer; ACaption: String);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);
   fFrameState:= fsRaised;
   fFrameWidth:= 2;
   fCaption:= ACaption;
End;
//********************************************************************
Procedure TGUIButton.Paint;
Begin
   If fFrameState = fsRaised
      Then FGColor.Gradient:= 0.7
      Else FGColor.Gradient:= 0.5;

   If fRoot.HoveredItem <> Self   // HOVER!
      Then fFrameState:= fsFlat;

   Inherited Paint;

   TextAlign:= taMidCenter;
   With fTextPos Do Font.Show(X,Y,fCaption);
End;
//********************************************************************
Procedure TGUIButton.MouseBtnLDown(X,Y,AddKeys: Integer);
Begin
   fFrameState:= fsLowered;
   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIButton.MouseBtnLUp(X,Y,AddKeys: Integer);
Begin
   fFrameState:= fsRaised;
   Inherited MouseBtnLUp(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIButton.MouseMove(X,Y: Integer);
Begin
   fFrameState:= fsRaised; // HOVER
   Inherited MouseMove(X,Y);
End;
//********************************************************************
//********************************************************************
//   T G U I S Y M B O L B T N
//********************************************************************
Procedure TGUISymbolBtn.Paint;
Begin
   Inherited Paint;
   fSymbol.Show(Left,Top,Width,Height,SOLIDGREY,fSpecification);
End;
//********************************************************************
//********************************************************************
//   T G U I S L I D E R
//********************************************************************
Constructor TGUISlider.Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer; AMovement: TMovement);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight,'');
   fDragMode:=0;
   fMovement:= AMovement;
End;
//********************************************************************
Procedure TGUISlider.Move(X,Y: Integer);
Begin
   Case fMovement Of
      Horizontal: Begin
         Y:= 0;
         If Left+X < fBorder1
            Then X:=fBorder1-Left;
         If Left+Width+X > fBorder2
            Then X:=fBorder2-Left-Width;
      End;
      Vertical: Begin
         X:= 0;
         If Top+Y < Border1
            Then Y:= Border1-Top;
         If Top+Height+Y > fBorder2
            Then Y:=Border2-Top-Height;
      End;
   End;
   Inherited Move(X,Y);
End;
//********************************************************************
Procedure TGUISlider.MouseBtnLDown(X,Y,AddKeys: Integer);
Begin
   fDragMode:= 1;
   StartX:=X; StartY:=Y;

   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUISlider.MouseBtnLUp(X,Y,AddKeys: Integer);
Begin
   fDragMode:= 0;
   Inherited MouseBtnLUp(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUISlider.MouseMove(X,Y: Integer);
Begin
   If fDragMode <> 0
      Then Begin
         Move(X-StartX,Y-StartY);
         StartX:=X; StartY:=Y;
      End;
   Inherited MouseMove(X,Y);
End;
//********************************************************************
//********************************************************************
//   T G U I P A N E L
//********************************************************************
Constructor TGUIPanel.Create(AName: String; AParent: TGUIItem;
                  ALeft,ATop,AWidth,AHeight: Integer);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);
   fFrameState:= fsRaised;
   //FGColor:= GraphTool.ColorByName['TRANSWHITE'];
End;
//********************************************************************
Procedure TGUIPanel.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;
End;
//********************************************************************
Procedure TGUIPanel.MouseBtnLDown(X,Y,AddKeys: Integer);
Begin
   fDragMode:= 1;
   If EdgeZoneX(X) Or EdgeZoneY(Y)
      Then fDragMode:= 2;

   StartX:=X; StartY:=Y;

   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIPanel.MouseBtnLUp(X,Y,AddKeys: Integer);
Begin
   fDragMode:= 0;
   Inherited MouseBtnLUp(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIPanel.MouseMove(X,Y: Integer);
Begin
   Case fDragMode Of
      1: Begin
            TREEMove(X-StartX,Y-StartY);
            StartX:=X; StartY:=Y;
         End;
      2: Resize(X,Y);
   End;
   Inherited MouseMove(X,Y);
End;
//********************************************************************
Procedure TGUIPanel.MouseWheelDown(X,Y,ADelta: Integer);
Begin
   Width:= Width+4;
   Height:= Height+4;
   Inherited MouseWheelDown(X,Y,ADelta);
End;
//********************************************************************
Procedure TGUIPanel.MouseWheelUp(X,Y,ADelta: Integer);
Begin
   Width:= Width-4;
   Height:= Height-4;
   Inherited MouseWheelUp(X,Y,ADelta);
End;
//********************************************************************
//********************************************************************
//   T G U I L A B E L
//********************************************************************
Constructor TGUILabel.Create(AName: String; AParent: TGUIItem;
                      ALeft,ATop,AWidth,AHeight: Integer;
                      ACaption: String);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);
   fFrameState:= fsFlat;
   fFrameWidth:= 1;
   fCaption:= ACaption;
   FGColor:= GraphTool.ColorByName['SOLIDWHITE'];
End;
//********************************************************************
Procedure TGUILabel.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;
   SetTextAlign(fTextAlign); // Font-Positioning
   With fTextPos Do Font.Show(X,Y,fCaption);
End;
//********************************************************************
//********************************************************************
//   T G U I C H E C K B O X
//********************************************************************
Constructor TGUICheckBox.Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer; ACaption: String);
Begin
   fBoxSize:= 18;
   fMargin:= 3;

   Inherited Create(AName,AParent,
      ALeft,ATop,AWidth,fBoxSize+2*fMargin);

   fLabel:= TGUILabel.Create(Name+'Label',Self,
      Left+fBoxSize+2*fMargin,Top,
      Width-fBoxSize-2*fMargin,Height,ACaption);

   fLabel.Shape:= Shape;
   fLabel.FGColor:= FGColor;
   fLabel.TextAlign:= taMidLeft;

   fBoxColor:= GraphTool.ColorByName['SolidWhite'];
   fChecked:= False;
   fHook:= GraphTool.SymbolByName['Hook'];
End;
//********************************************************************
Procedure TGUICheckBox.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;
   Shape.Show(Left+fMargin,Top+fMargin,fBoxSize,fBoxSize,dmAll,
      fBoxColor,Image,2,fsLowered);
   If fChecked
      Then fHook.Show(Left+fMargin,Top+fMargin,fBoxSize,fBoxSize,
                                   SolidGrey,0);
End;
//********************************************************************
Procedure TGUICheckBox.SetAlign(Align: TAlignment);
Begin
   Inherited SetAlign(Align);

   fLabel.Left:= Left+fBoxSize+2*fMargin;
   fLabel.Top:= Top;
   fLabel.Width:= Width-fBoxSize-2*fMargin;
   fLabel.Height:= Height;
End;
//********************************************************************
Procedure TGUICheckBox.MouseBtnLDown(X,Y,AddKeys: Integer);
Begin
   fChecked:= Not fChecked;
   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
//********************************************************************
//   T G U I S C R O L L B A R
//********************************************************************
// fSlider has to be a SHAPE because if it was a button, it would
// grab the mouseclick and execute its own visual reaction first
Constructor TGUIScrollBar.Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);

   FrameWidth:=1;
   TextAlign:= taMidCenter;

   fMin         :=0;
   fMax         :=100;
   fLargeChange :=20;
   fSmallChange :=5;
   fPosition    :=0;

   fFirstBtn:= TGUISymbolBtn.Create(Name+'FirstBtn',Self,0,0,0,0,'');
   fFirstBtn.Symbol:= GraphTool.SymbolByName['Arrow'];

   fSecondBtn:= TGUISymbolBtn.Create(Name+'SecondBtn',Self,0,0,0,0,'');
   fSecondBtn.Symbol:= GraphTool.SymbolByName['Arrow'];

   CreateButtons;
   SetAll;

   fFirstBtn.FGColor:= FGColor;
   fFirstBtn.OnMouseBtnLUp:= Decrease;
   fSecondBtn.FGColor:= FGColor;
   fSecondBtn.OnMouseBtnLUp:= Increase;
   fSlider.FGColor:= FGColor;
   fSlider.Shape:= Shape;
   fSlider.OnMouseMove:= SliderShift;
End;
//********************************************************************
Procedure TGUIScrollBar.Paint;
Begin
   If fRoot.HoveredItem <> Self   // HOVER!
      Then fFrameState:= fsFlat;

   FGColor.Reset;
   Inherited Paint;

   SetTextAlign(fTextAlign); // Font-Positioning
   With fTextPos Do Font.Show(X,Y,IntToStr(fPosition));
End;
//********************************************************************
Procedure TGUIScrollBar.Move(X,Y: Integer);
Begin
   Inherited Move(X,Y);
   SetAll;
End;
//********************************************************************
Procedure TGUIScrollBar.SetAlign(Align: TAlignment);
Begin
   Inherited SetAlign(Align);
   If fParent <> Nil
      Then SetAll;
End;
//********************************************************************
Procedure TGUIScrollBar.Decrease(X,Y,AddKeys: Integer);
Var fStepSmall: Integer;
Begin
   With fSlider Do
      fStepSmall:= Trunc(fSmallChange/(fMax-fMin)*(Border2-Border1));

   fSlider.Move(-fStepSmall,-fStepSmall);
   CalcPosition;
   Scroll(scLineUp,fPosition);
End;
//********************************************************************
Procedure TGUIScrollBar.Increase(X,Y,AddKeys: Integer);
Var fStepSmall: Integer;
Begin
   With fSlider Do
      fStepSmall:= Trunc(fSmallChange/(fMax-fMin)*(Border2-Border1));

   fSlider.Move(+fStepSmall,+fStepSmall);
   CalcPosition;
   Scroll(scLineDown,fPosition);
End;
//********************************************************************
Procedure TGUIScrollBar.SetPosition(APosition: Integer);
Begin
   If (APosition >= fMin) And (APosition <= fMax)
      Then Begin
         fPosition:= APosition Div(fMax-fMin);
      End;
End;
//********************************************************************
Procedure TGUIScrollBar.SetMax(AMax: Integer);
Begin
   If AMax > fMin
      Then fMax:= AMax;
   SetSlider;
End;
//********************************************************************
Procedure TGUIScrollBar.SetMin(AMin: Integer);
Begin
   fMin:= AMin;
   SetSlider;
End;
//********************************************************************
Procedure TGUIScrollBar.SetLargeChange(ALargeChange: Integer);
Begin
   If (ALargeChange > 0) And (ALargeChange <= fMax)
      Then Begin
         fLargeChange:= ALargeChange;
      End;
   SetSlider;
End;
//********************************************************************
Procedure TGUIScrollBar.SetSmallChange(ASmallChange: Integer);
Begin
   If (ASmallChange > 0) And (ASmallChange <= fMax)
      Then Begin
         fSmallChange:= ASmallChange;
      End;
End;
//********************************************************************
Procedure TGUIScrollBar.MouseMove(X,Y: Integer);
Begin
   fFrameState:= fsRaised; // HOVER
   Inherited MouseMove(X,Y);
End;
//********************************************************************
Procedure TGUIScrollBar.Scroll(AScrollCode: TScrollCode;
                      AScrollPos: Integer);
Begin
   If Assigned(fOnScroll)
      Then fOnScroll(AScrollCode,AScrollPos);
End;
//********************************************************************
Procedure TGUIScrollBar.SliderShift(X,Y: Integer);
Begin
   CalcPosition;
   Scroll(scPosition,fPosition);
End;
//********************************************************************
//********************************************************************
//   T G U I S C R O L L H O R I Z B A R
//********************************************************************
Procedure TGUIScrollHorizBar.CalcPosition;
Begin
   With fSlider Do
      fPosition:= Trunc
         ((Left-Border1)/(Border2-Border1-Width)*(fMax-fMin));
End;
//********************************************************************
Procedure TGUIScrollHorizBar.CreateButtons;
Begin
   fFirstBtn.Specification:= 2;
   fSecondBtn.Specification:= 0;

   fSlider:= TGUISlider.Create(Name+'Slider',Self,0,0,0,0,Horizontal);
End;
//********************************************************************
Procedure TGUIScrollHorizBar.MouseBtnLDown(X,Y,AddKeys: Integer);
Var fStepLarge: Integer;
Begin
   With fSlider Do Begin
      fStepLarge:= Trunc(fLargeChange/(fMax-fMin)*(Border2-Border1));

      If X > Left+Width
         Then Begin
            fSlider.Move(fStepLarge,fStepLarge);
            Scroll(scPageDown,fPosition);
         End;
      if X < Left
         Then Begin
            fSlider.Move(-fStepLarge,-fStepLarge);
            Scroll(scPageUp,fPosition);
         End;
   End;
   CalcPosition;

   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIScrollHorizBar.SetAll;
Begin
   fFirstBtn.Left   := Left+FrameWidth;
   fFirstBtn.Top    := Top+FrameWidth;
   fFirstBtn.Width  := Height-2*FrameWidth;
   fFirstBtn.Height := Height-2*FrameWidth;

   fSecondBtn.Left   := Left+Width-Height+FrameWidth;
   fSecondBtn.Top    := Top+FrameWidth;
   fSecondBtn.Width  := Height-2*FrameWidth;
   fSecondBtn.Height := Height-2*FrameWidth;

   fSlider.Border1:= Left+fFrameWidth+Height;
   fSlider.Border2:= Left+fFrameWidth+Width-Height;

   fSlider.Top:= Left+FrameWidth;
   fSlider.Height:= Height-2*fFrameWidth;

   SetSlider;
End;
//********************************************************************
Procedure TGUIScrollHorizBar.SetSlider;
Begin
   With fSlider Do Begin
      Width:= Trunc(fLargeChange/(fMax-fMin)*(Border2-Border1));
      Left:= Trunc(fPosition/(fMax-fMin)*(Border2-Border1-Width)+Border1);
   End;
End;
//********************************************************************
//********************************************************************
//   T G U I S C R O L L V E R T B A R
//********************************************************************
Procedure TGUIScrollVertBar.CalcPosition;
Begin
   With fSlider Do
      fPosition:= Trunc
         ((Top-Border1)/(Border2-Border1-Height)*(fMax-fMin));
End;
//********************************************************************
Procedure TGUIScrollVertBar.CreateButtons;
Begin
   fFirstBtn.Specification:= 1;
   fSecondBtn.Specification:= 3;

   fSlider:= TGUISlider.Create(Name+'Slider',Self,0,0,0,0,Vertical);
End;
//********************************************************************
Procedure TGUIScrollVertBar.MouseBtnLDown(X,Y,AddKeys: Integer);
Var fStepLarge: Integer;
Begin
   With fSlider Do Begin
      fStepLarge:= Trunc(fLargeChange/(fMax-fMin)*(Border2-Border1));

      If Y > Top+Height
         Then Begin
            fSlider.Move(fStepLarge,fStepLarge);
            Scroll(scPageDown,fPosition);
         End;
      if Y < Top
         Then Begin
            fSlider.Move(-fStepLarge,-fStepLarge);
            Scroll(scPageUp,fPosition);
         End;
   End;
   CalcPosition;

   Inherited MouseBtnLDown(X,Y,AddKeys);
End;
//********************************************************************
Procedure TGUIScrollVertBar.SetAll;
Begin
   fFirstBtn.Left   := Left+FrameWidth;
   fFirstBtn.Top    := Top+FrameWidth;
   fFirstBtn.Width  := Width-2*FrameWidth;
   fFirstBtn.Height := Width-2*FrameWidth;

   fSecondBtn.Left   := Left+FrameWidth;
   fSecondBtn.Top    := Top+Height-Width+FrameWidth;
   fSecondBtn.Width  := Width-2*FrameWidth;
   fSecondBtn.Height := Width-2*FrameWidth;

   fSlider.Border1   := Top+fFrameWidth+Width;
   fSlider.Border2   := Top+fFrameWidth+Height-Width;

   fSlider.Left:= Left+FrameWidth;
   fSlider.Width:= Width-2*fFrameWidth;

   SetSlider;
End;
//********************************************************************
Procedure TGUIScrollVertBar.SetSlider;
Begin
   With fSlider Do Begin
      Height:= Trunc(fLargeChange/(fMax-FMin)*(Border2-Border1));
      Top:= Trunc(fPosition/(fMax-fMin)*(Border2-Border1-Height)+Border1);
   End;
End;
//********************************************************************
//********************************************************************
//   T G U I P R O G R E S S B A R
//********************************************************************
Constructor TGUIProgressBar.Create(AName: String; AParent: TGUIItem;
         ALeft,ATop,AWidth,AHeight: Integer);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);

   TextAlign:= taMidCenter;
   fColor:= GraphTool.ColorByName['SolidBlue'];
End;
//********************************************************************
Procedure TGUIProgressBar.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;
   Shape.Show(Left,Top,Trunc(Width*Value),Height,dmShapeOnly,
      fColor,Image,0,fsFlat);
   SetTextAlign(fTextAlign); // Font-Positioning
   With fTextPos Do Font.Show(X,Y,IntToStr(Trunc(fValue*100))+'%');
End;
//********************************************************************
Procedure TGUIProgressBar.SetValue(Value: Single);
Begin
   fValue:= Value;
   If fValue > 1
      Then fValue:= 1;
   If fValue < 0
      Then fValue:= 0;
End;
//********************************************************************
//********************************************************************
//   T G U I L I S T B O X
//********************************************************************
Constructor TGUIListBox.Create(AName: String; AParent: TGUIItem;
   ALeft,ATop,AWidth,AHeight: Integer);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);
   fItems:= TStringList.Create;
   fItemIndex:= -1;
   fTopItem:= -1;
   fFrameState:= fsLowered;
   fFrameWidth:= 2;
   FGColor:= GraphTool.ColorByName['SolidLGrey'];
   fMargin:= 10;
   fLineHeight:= Font.TextHeight('X');
   fLinesPerPage:= (Height-fMargin) Div fLineHeight;

   fScrollBar:= TGUIScrollVertBar.Create(Name+'ScrollBar',Self,
              Left+Width-20,Top,20,Height);
   fScrollBar.Align:= alRight;
   fScrollBar.OnScroll:= TrackPosition;
End;
//********************************************************************
Destructor TGUIListBox.Destroy;
Begin
   fItems.Free;
   Inherited Destroy;
End;
//********************************************************************
Procedure TGUIListBox.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;

   If (fItems.Count > 0) And (fTopItem >= 0)
      Then Begin
         ShowPage;
      End;
End;
//********************************************************************
Function TGUIListBox.GetItem(AIndex: Integer): String;
Begin
   Result:='';
   If ValidIndex(AIndex)
      Then Result:= fItems[AIndex];
End;
//********************************************************************
Procedure TGUIListBox.SetItem(AIndex: Integer; AString: String);
Begin
   If ValidIndex(AIndex)
      Then fItems.Insert(AIndex,AString)
      Else fItems.Add(AString);

   With fScrollBar Do Begin
      fMin:= 0;
      fMax:= fMargin + fItems.Count*fLineHeight;
      fLargeChange:= Height;
      fSmallChange:= fLineHeight;
      fPosition:=0;
   End;
   If fTopItem < 0
      Then fTopItem:=0;
End;
//********************************************************************
Procedure TGUIListBox.ShowPage;
Var X,Y,I: Integer;
Begin
   If fTopItem >= 0 Then Begin
      X:= Left+fMargin;
      Y:= Top+fMargin;

      I:= fTopItem;
      With fItems Do
         While (I < Count) And (Y < Top+Height-fLineHeight-fMargin)
            Do Begin
               If fItemIndex = I
                  Then Font.RGBAColor:= SolidRed;
               Font.Show(X,Y,Items[I]);
               If fItemIndex = I
                  Then Font.RGBAColor:= SolidBlack;
               Inc (Y, fLineHeight);
               Inc(I); //==========>
            End;
   End;
End;
//********************************************************************
Procedure TGUIListBox.TrackPosition(AScrollCode: TScrollCode;
                    AScrollPos: Integer);
Begin
   If fItems.Count > 0 Then Case ASCrollCode Of
        scLineUp: If fTopItem > 0 Then Dec(fTopItem);
      scLineDown: If fTopItem < fItems.Count-1 Then Inc(fTopItem);
        scPageUp: If fTopItem > fLinesPerPage-1
                     Then Dec(fTopItem,fLinesPerPage);
      scPageDown: If fTopItem + fLinesPerPage < fItems.Count-1
                     Then Inc(fTopItem,fLinesPerPage);
      scPosition: fTopItem:= (AScrollPos-fMargin) Div fLineHeight;
   End;
End;
//********************************************************************
Function TGUIListBox.ValidIndex(AIndex: Integer): Boolean;
Begin
   Result:= False;
   With fItems Do If Count > 0
      Then If (AIndex >= 0) And (AIndex < Count)
         Then Result:= True;
End;
//********************************************************************
Procedure TGUIListBox.MouseBtnLDown(X,Y,AddKeys: Integer);
Begin
   fItemIndex:= Trunc((Y-Top-fMargin)/fLineHeight)+ fTopItem;
End;
//********************************************************************
//********************************************************************
//   T G U I C O M B O B O X
//********************************************************************
Constructor TGUIComboBox.Create(AName: String; AParent: TGUIItem;
   ALeft,ATop,AWidth,AHeight: Integer);
Begin
   Inherited Create(AName,AParent,ALeft,ATop,AWidth,AHeight);
   fItems:= TStringList.Create;
   fItemIndex:= -1;
   fScrollBar:= TGUIScrollVertBar.Create(Name+'ScrollBar',Self,
              Left+Width-25,Top,25,Height);
   fScrollBar.Align:= alRight;
   FrameWidth:= 3;
End;
//********************************************************************
Destructor TGUIComboBox.Destroy;
Begin
   fItems.Free;
   Inherited Destroy;
End;
//********************************************************************
Procedure TGUIComboBox.Paint;
Begin
   FGColor.Reset;
   Inherited Paint;
End;
//********************************************************************
//********************************************************************
End.
