//********************************************************************
UNIT OSSupport;
//********************************************************************
INTERFACE

USES
   Messages,
   Windows;

CONST
   KEY_SHIFT = 256;
   KEY_CONTROL = 257;
   KEY_ALT = 258;
   KEY_INSERT = 259;
   KEY_DELETE = 260;
   KEY_F4 = 261;

   ELLIPSEFACTOR = 0.2;

TYPE
   TIntRec = Record Low,High: Smallint; End;
   TVideoMode = (vmNormal,vmFullscreen);
   TMouseImage = (miArrow, miMoving, miSizing, miHourGlass);

   TWndChangeEvent = Procedure(AReceiver: LongWord; X,Y: Integer) Of Object;
   TWndNotifyEvent = Procedure(AReceiver: LongWord) Of Object;
   TWndKeyEvent = Procedure(AReceiver: LongWord; AKey,PreviousKey: LongWord) Of Object;
   TWndMouseBtnEvent = Procedure(AReceiver: LongWord; X,Y,AddKeys: Integer) Of Object;
   TWndMouseMoveEvent = Procedure(AReceiver: LongWord; X,Y: Integer) Of Object;
   TWndMouseWheelEvent = Procedure(AReceiver: LongWord; X,Y,ADelta: Integer) Of Object;

   TEventPipeline = Record
      OnPaint         : TWndNotifyEvent;
      OnMove          : TWndChangeEvent;
      OnResize        : TWndChangeEvent;

      OnKeyDown       : TWndKeyEvent;
      OnKeyUp         : TWndKeyEvent;
      OnMouseBtnLDown : TWndMouseBtnEvent;
      OnMouseBtnLUp   : TWndMouseBtnEvent;
      OnMouseBtnRDown : TWndMouseBtnEvent;
      OnMouseBtnMDown : TWndMouseBtnEvent;
      OnMouseMove     : TWndMouseMoveEvent;
      OnMouseWheelDown: TWndMouseWheelEvent;
      OnMouseWheelUp  : TWndMouseWheelEvent;
   End;

//********************************************************************
Procedure EventTerminateApplication;
Function  EventDispatch: Boolean;
Function  EventReceive(AWindow: LongWord; AMessage,
              WParam,LParam: Longint): Integer; Stdcall; Export;
Function  KeyCode(AKey: Longint): LongInt;
//--------------------------------------------------------------------
Procedure MouseCaptureRelease;
Function  MouseClientToScreen(AWindow: LongWord; AX,AY: Integer): TPoint;
Procedure MouseImageSet(AWindow: LongWord; AMouseImage: TMouseImage;
                              MouseX,MouseY: Integer);
//--------------------------------------------------------------------
Function  RenderContextCreate(AWindow: LongWord): LongWord;
Function  RenderContextFree(AWindow,
                      ARenderingContext: LongWord): Boolean;
Function  RenderContextActivate(AWindow,
                      ARenderingContext: LongWord): Boolean;
Procedure RenderContextDeactivate;
//--------------------------------------------------------------------
//Procedure VideoModeSet(AVideoMode: TVideoMode);
//--------------------------------------------------------------------
Procedure WindowClassCreate(AWindowClassName: String);
Procedure WindowClassDelete(AWindowClassName: String);
Function  WindowCreate(AWindowClassName,ATitel: String;
                 ALeft,ATop,AWidth,AHeight: Integer): LongWord;
Procedure WindowDestroy(AWindow,ARegion: LongWord);
Procedure WindowMaximize(AWindow: LongWord);
Procedure WindowMinimize(AWindow: LongWord);
Procedure WindowMove(AWindow: LongWord; ALeft,ATop: Integer);
Procedure WindowMoveByMouse(AWindow: LongWord);
Function  WindowRect(AWindow: LongWord): TRect;
Function  WindowRegion(AWindow: LongWord; AWidth,AHeight: Integer): LongWord;
Procedure WindowResize(AWindow: LongWord; AWidth,AHeight: Integer);
Procedure WindowResizeByMouse(AWindow: LongWord);
Procedure WindowShow(AWindow: LongWord; Visible: Boolean);
Procedure WindowSwapBuffers(AWindow: LongWord);
//********************************************************************
VAR
   EventPipeline: TEventPipeline;
//********************************************************************
IMPLEMENTATION

Var
   PreviousKey: LongWord;
//********************************************************************
//********************************************************************
//   E V E N T S
//********************************************************************
Function  EventDispatch: Boolean;
Var
   Msg: TMsg;
Begin
   Result:= False;

   If PeekMessage(Msg, 0, 0, 0, PM_REMOVE)
      Then Begin
         TranslateMessage(Msg);
         If Msg.wParam <> VK_F4  // Disables ALT+F4 :)
            Then DispatchMessage(Msg); 
         Result:= True;
      End;
End;
//********************************************************************

//********************************************************************
Function EventReceive(AWindow: LongWord; AMessage,
                   WParam,LParam: LongInt): Integer;
Var
   Longint1,Longint2,Longint3: Longint;
   Rect: TRect;
Begin
   If AMessage = WM_SYSCOMMAND Then Begin
      Case WParam Of
         SC_MOVE Or HTCAPTION:;
         SC_SIZE Or WMSZ_BOTTOMRIGHT:;
      End;
   End;

   With EventPipeline Do Case AMessage Of

      WM_PAINT: Begin
         If Assigned(OnPaint)
            Then OnPaint(AWindow);
         // This stops the WM_PAINT-loop
         GetUpdateRect(AWindow,Rect,True);
         ValidateRect(AWindow,@Rect);

         Result:= 0;
      End;

      WM_SIZE: Begin
         Result:= 0;
      End;

      WM_SETCURSOR: Begin
         Result:= 0;
      End;

      WM_KEYDOWN: Begin
         If Assigned(OnKeyDown) Then Begin
            WParam:= KeyCode(WParam);
            OnKeyDown(AWindow, WParam, PreviousKey);
            PreviousKey:= WParam;
         End;
         Result:= 0;
      End;

      WM_KEYUP: Begin
         If Assigned(OnKeyUp) Then Begin
            OnKeyUp(AWindow, WParam,0);
         End;
         Result:= 0;
      End;

      WM_LBUTTONDOWN: Begin
         If Assigned(OnMouseBtnLDown) Then Begin
            Longint1:= TIntRec(LParam).Low;
            Longint2:= TIntRec(LParam).High;
            If WParam = WParam Or MK_CONTROL
               Then WParam:= KEY_CONTROL;
            OnMouseBtnLDown(AWindow, Longint1,Longint2,WParam);
         End;
         Result:= 0;
      End;

      WM_LBUTTONUP: Begin
         If Assigned(OnMouseBtnLUp) Then Begin
            Longint1:= TIntRec(LParam).Low;
            Longint2:= TIntRec(LParam).High;
            OnMouseBtnLUp(AWindow, Longint1,Longint2,WParam);
         End;
         Result:= 0;
      End;

      WM_RBUTTONDOWN: Begin
         If Assigned(OnMouseBtnRDown) Then Begin
            Longint1:= TIntRec(LParam).Low;
            Longint2:= TIntRec(LParam).High;
            OnMouseBtnRDown(AWindow, Longint1,Longint2,WParam);
         End;
         Result:= 0;
      End;

      WM_MBUTTONDOWN: Begin
         If Assigned(OnMouseBtnMDown) Then Begin
            Longint1:= TIntRec(LParam).Low;
            Longint2:= TIntRec(LParam).High;
            OnMouseBtnMDown(AWindow, Longint1,Longint2,WParam);
         End;
         Result:= 0;
      End;

      WM_MOUSEMOVE: Begin
         If Assigned(OnMouseMove) Then Begin
            Longint1:= TIntRec(LParam).Low;
            Longint2:= TIntRec(LParam).High;
            OnMouseMove(AWindow, Longint1,Longint2);
         End;
         Result:= 0;
      End;

      WM_MOUSEWHEEL: Begin
         LongInt3:= TIntRec(WParam).High;
         If LongInt3 > 0
            Then Begin
               If Assigned(OnMouseWheelUp) Then Begin
                  Longint1:= TIntRec(LParam).Low;
                  Longint2:= TIntRec(LParam).High;
                  OnMouseWheelUp
                     (AWindow, Longint1,Longint2,Longint3);
               End;
            End
            Else Begin
               If Assigned(OnMouseWheelDown) Then Begin
                  Longint1:= TIntRec(LParam).Low;
                  Longint2:= TIntRec(LParam).High;
                  OnMouseWheelDown(AWindow,
                     Longint1,Longint2,Longint3);
               End;
            End;
         Result:= 0;
      End;

      Else
         Result:= DefWindowProc(AWindow,AMessage,WParam,LParam);

   End; // Of Case
End;
//********************************************************************
Procedure EventTerminateApplication;
Begin
   PostQuitMessage(0);
End;
//********************************************************************
Function KeyCode(AKey: Longint): LongInt;
Begin
   Result:= AKey;

   Case AKey Of
      VK_SHIFT   : Result:= KEY_SHIFT;
      VK_CONTROL : Result:= KEY_CONTROL;
      VK_MENU    : Result:= KEY_ALT;
      VK_INSERT  : Result:= KEY_INSERT;
      VK_DELETE  : Result:= KEY_DELETE;
      VK_F4      : Result:= KEY_F4;
   End;
End;
//********************************************************************
//********************************************************************
//   M O U S E
//********************************************************************
Procedure MouseCaptureRelease;
Begin
   ReleaseCapture;
End;
//********************************************************************
Function MouseClientToScreen(AWindow: LongWord; AX,AY: Integer): TPoint;
Begin
   With Result Do Begin X:= AX; Y:= AY; End;
   ClientToScreen(AWindow,Result);
End;
//********************************************************************
Procedure MouseImageSet(AWindow: LongWord; AMouseImage: TMouseImage;
                           MouseX,MouseY: Integer);
Var
   MouseCursor: hCursor;
Begin
   MouseCursor:= LoadCursor(0,idc_Arrow);
   Case AMouseImage Of
      miMoving    : MouseCursor:= LoadCursor(0,idc_SizeAll);
      miSizing    : MouseCursor:= LoadCursor(0,idc_SizeNWSE);
      miHourGlass : MouseCursor:= LoadCursor(0,idc_Wait);
   End;
   SetCursor(MouseCursor);
   //SetClassLong(AWindow, gcl_hCursor, MouseCursor);
End;
//********************************************************************
//   R E N D E R C O N T E X T
//********************************************************************
Function RenderContextCreate(AWindow: LongWord): LongWord;
Var
   DeviceContext: HDC;
   RenderingContext: HGLRC;
   PixelFormatRec: PIXELFORMATDESCRIPTOR;
   PixelFormat: Integer;
Begin
   FillChar(PixelFormatRec, SizeOf(PixelFormatRec), 0);

   With PixelFormatRec Do Begin
      nSize      := SizeOf(PixelFormatRec);
      nVersion   := 1;
      dwFlags    := PFD_SUPPORT_OPENGL;
      dwFlags    := dwFlags OR PFD_DRAW_TO_WINDOW;
      dwFlags    := dwFlags OR PFD_DOUBLEBUFFER;
      iPixelType := PFD_TYPE_RGBA;
      cColorBits := 32;
      cDepthBits := 24;
      iLayerType := PFD_MAIN_PLANE;
   End;

   DeviceContext:= GetDC(AWindow);

   PixelFormat := ChoosePixelFormat(DeviceContext, @PixelFormatRec);
   SetPixelFormat(DeviceContext, PixelFormat, @PixelFormatRec);

   RenderingContext:= wglCreateContext(DeviceContext);
   wglMakeCurrent(DeviceContext, RenderingContext);
   Result:= RenderingContext;
End;
//********************************************************************
Function  RenderContextFree(AWindow,
                      ARenderingContext: LongWord): Boolean;
Begin
   Result:= False;

   If RenderContextActivate(AWindow,ARenderingContext)
      Then Begin
         wglDeleteContext(ARenderingContext);
         ReleaseDC(AWindow, GetDC(AWindow));
         Result:= True;
      End;
End;
//********************************************************************
Function RenderContextActivate(AWindow,
                  ARenderingContext: LongWord): Boolean;
Var
   DeviceContext: HDC;
Begin
   Result:= False;

   DeviceContext:= GetDC(AWindow);

   If (DeviceContext <> 0) And (ARenderingContext <> 0)
      Then Begin
         wglMakeCurrent(GetDC(AWindow),ARenderingContext);
         Result:= True;
      End;
End;
//********************************************************************
Procedure RenderContextDeactivate;
Begin
  wglMakeCurrent(0, 0);
End;
//********************************************************************
//   V I D E O M O D E
//********************************************************************
Procedure VideoModeSet(AVideoMode: TVideoMode);
Var ScreenSettings: DevMode;
    ResultMode,ScreenWidth,ScreenHeight,BitsPerPixel,
    Frequency: Integer;
Begin
   If AVideoMode = vmFullScreen
      Then Begin
         ScreenWidth  :=  640;
         ScreenHeight :=  480;
      End
      Else Begin
         ScreenWidth  := 1024;
         ScreenHeight :=  768;
      End;
      BitsPerPixel :=  32;
      Frequency    :=  60;

   With ScreenSettings Do Begin
      dmSize := SizeOf(ScreenSettings);
      dmPelsWidth  := ScreenWidth;
      dmPelsHeight := ScreenHeight;
      dmBitsPerPel := BitsPerPixel;
      dmDisplayFrequency := Frequency;
      dmFields := DM_PELSWIDTH Or DM_PELSHEIGHT Or
                  DM_BITSPERPEL Or DM_DISPLAYFREQUENCY;
   End;

   If AVideoMode = vmFullScreen
      Then ResultMode:= ChangeDisplaySettings
                     (ScreenSettings, CDS_FULLSCREEN Or CDS_RESET)
      Else ResultMode:= ChangeDisplaySettings(ScreenSettings, 0);

   If ResultMode = DISP_CHANGE_FAILED
      Then; //???
End;
//********************************************************************
//   W I N D O W S
//********************************************************************
Procedure WindowClassCreate(AWindowClassName: String);
Var
   WindowClass: WndClass;
Begin
   With WindowClass Do Begin
      Style := CS_OWNDC;
      lpfnWndProc := @EventReceive;
      cbClsExtra := 0;
      cbWndExtra := 0;
      hInstance := System.MainInstance;
      hIcon := LoadIcon(0, idi_Application);
      hCursor := LoadCursor(0, idc_Arrow);
      //hbrBackground := GetStockObject(WHITE_BRUSH);
      hbrBackground := 0;
      lpszMenuName := Nil;
      lpszClassName := PChar(AWindowClassName);
   End;

   RegisterClass(WindowClass);
End;
//********************************************************************
Procedure WindowClassDelete(AWindowClassName: String);
Begin
   UnregisterClass(PChar(AWindowClassName),System.MainInstance);
End;
//********************************************************************
Function WindowCreate(AWindowClassName,ATitel: String;
                 ALeft,ATop,AWidth,AHeight: Integer): LongWord;
Const
   ParentWindow = 0;
   MainMenu = 0;
Var
   WinStyle: LongWord;
Begin
  //WinStyle:= WS_EX_TRANSPARENT;
  //WinStyle:= WS_BORDER;
  WinStyle:= WS_POPUPWINDOW;

  Result:= CreateWindow(PChar(AWindowClassName), PChar(ATitel),
           WinStyle, ALeft,ATop,AWidth,AHeight,
           ParentWindow, MainMenu, hInstance, Nil);
End;
//********************************************************************
Procedure WindowDestroy(AWindow,ARegion: LongWord);
Begin
   If ARegion <> 0 Then Begin
      SetWindowRgn(AWindow, 0, TRUE); // Release region
      DeleteObject(ARegion);     // Release GDI-Object
   End;
   DestroyWindow(AWindow);
End;
//********************************************************************
Procedure WindowMaximize(AWindow: LongWord);
Begin
   ShowWindow(AWindow,SW_MAXIMIZE);
End;
//********************************************************************
Procedure WindowMinimize(AWindow: LongWord);
Begin
   ShowWindow(AWindow,SW_MINIMIZE);
End;
//********************************************************************
Procedure WindowMove(AWindow: LongWord; ALeft,ATop: Integer);
Begin
   SetWindowPos(AWindow,HWND_TOP,ALeft,ATop,0,0,SWP_NOSIZE);
End;
//********************************************************************
Procedure WindowMoveByMouse(AWindow: LongWord);
Begin
   SendMessage(AWindow,WM_SYSCOMMAND,SC_MOVE Or HTCAPTION,0);
End;
//********************************************************************
Function WindowRect(AWindow: LongWord): TRect;
Var WindowRectangle: TRect;
Begin
   GetWindowRect(AWindow,WindowRectangle);
   Result:= WindowRectangle;
End;
//********************************************************************
Function WindowRegion(AWindow: LongWord; AWidth,AHeight: Integer): LongWord;
Var WindowRegion: LongWord;
Begin
  WindowRegion:= CreateRectRgn(0,0,AWidth,AHeight);

  {WindowRegion:= CreateRoundRectRgn(0,0,AWidth,AHeight,
                  Trunc(AWidth*EllipseFactor),
                  Trunc(AHeight*EllipseFactor));  }

  If WindowRegion <> 0
     Then SetWindowRgn(AWindow,WindowRegion,TRUE);

  Result:= WindowRegion;
End;
//********************************************************************
Procedure WindowResize(AWindow: LongWord; AWidth,AHeight: Integer);
Begin
   SetWindowPos(AWindow,HWND_TOP,0,0,AWidth,AHeight,SWP_NOMOVE);
End;
//********************************************************************
Procedure WindowResizeByMouse(AWindow: LongWord);
Begin
   SendMessage(AWindow,WM_SYSCOMMAND,SC_SIZE Or WMSZ_BOTTOMRIGHT, 0);
End;
//********************************************************************
Procedure WindowShow(AWindow: LongWord; Visible: Boolean);
Begin
   If Visible
      Then ShowWindow(AWindow,SW_SHOW)
      Else ShowWindow(AWindow,SW_HIDE);
End;
//********************************************************************
Procedure WindowSwapBuffers(AWindow: LongWord);
Begin
   SwapBuffers(GetDC(AWindow));
End;
//********************************************************************
//********************************************************************

INITIALIZATION

PreviousKey:= 0;

End.
