//********************************************************************
UNIT Render3D;
//********************************************************************
INTERFACE

Uses
  SysUtils,
  dglOpenGl,
  OSSupport,
  GraphicSupport,
  GUI,
  Windows;

Procedure RenderGradientsOnPaint(AWindow: TGUIWindow);
Procedure RenderQuadOnIdle(AWindow: TGUIWindow);
Procedure RenderQuadOnPaint(AWindow: TGUIWindow);

Var
   LocalRenderContext: LongWord;
   MyColor: TRGBAColor;
   MyImage: TRGBAImage;
   MyFont:  TFreeType2Font;
//********************************************************************
IMPLEMENTATION
//********************************************************************
Var
   Angle: Integer;
   StartTime,TimeCount,FrameCount,FPS: Integer;
//********************************************************************
Procedure RenderSmallQuad;
Begin
   glBegin(GL_QUADS);
      glTexCoord2f(0,0); glVertex2f( 0,20); // Links unten
      glTexCoord2f(1,0); glVertex2f(20,20); // Rechts unten
      glTexCoord2f(1,1); glVertex2f(20, 0); // Rechts oben
      glTexCoord2f(0,1); glVertex2f( 0, 0); // Links oben
   glEnd;
End;
//********************************************************************
Procedure RenderGradientsOnPaint(AWindow: TGUIWindow);
//--------------------------------------------------
Procedure ShowGradients(AGUIColor: TRGBAColor);
Var
   I: Integer;
Begin
   With AGUIColor,RGBAColor Do Begin
      Reset; // Reset color
      glTranslatef(0,25,0);
      glColor4ub(Red,Green,Blue,Alpha);
      RenderSmallQuad;

      glPushMatrix;
      For I:= 1 To 20 Do Begin
         glTranslatef(20,0,0);
         Gradient:= 0.05 * I;
         glColor4ub(Red,Green,Blue,Alpha);
         RenderSmallQuad;
      End;
      glPopMatrix;
   End;
End;
//--------------------------------------------------
Var
   Viewport: Array [0..3] Of Integer;
Begin
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glGetIntegerv(GL_VIEWPORT, @Viewport);
   glOrtho(0,ViewPort[2],ViewPort[3],0,+1,-1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;

   {With AWindow.GraphTool Do Begin
      ShowGradients(ColorByName['SOLIDGREY']);
      ShowGradients(ColorByName['SOLIDRED']);
      ShowGradients(ColorByName['SOLIDGREEN']);
      ShowGradients(ColorByName['SOLIDBLUE']);
   End; }

   // Schrifttextur ----------------------------------------
   glBindTexture(GL_TEXTURE_2D, 1);
   With SOLIDBLUE Do glColor4ub(Red,Green,Blue,Alpha);
   With AWindow Do Begin
      glBegin(GL_QUADS);
         glTexCoord2f(0,0); glVertex2f(Left ,Top);
         glTexCoord2f(1,0); glVertex2f(Width,Top);
         glTexCoord2f(1,1); glVertex2f(Width,Height);
         glTexCoord2f(0,1); glVertex2f(Left ,Height);
      glEnd;
   End;
   glBindTexture(GL_TEXTURE_2D, 0);
   // ------------------------------------------------------
End;
//********************************************************************
Procedure RenderQuad(AWindow: TGUIWindow);
Var
   Viewport: Array [0..3] Of Integer;
   X,Y,Z: Single;
Begin
   //glScissor(100,100,100,100);
   //glEnable(GL_SCISSOR_TEST);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glGetIntegerv(GL_VIEWPORT, @viewport);
   X:= ViewPort[2];
   Y:= ViewPort[3];
   gluPerspective(45,X/Y,0.1,1000);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;

   If Angle >= 360
      Then Angle:= 0
      Else Inc(Angle,1);

   X:= sin(Angle*PI/180)*400;
   Z:= cos(Angle*PI/180)*400;
   glRotatef(-Angle,0,1,0);    // Weltrotation
   glTranslatef(-X,0,-Z);      // Verschiebung der rotierten Welt
                               // in Abhngigkeit von der Rotation
   glRotatef(Angle,1,0,0);     // Objektrotation

   With MyColor Do Begin
      Reset; Show;
   End;
   MyImage.Show(TRUE);
   glBegin(GL_QUADS);
      glTexCoord2f(0,0); glVertex2f(-100,-100);
      glTexCoord2f(1,0); glVertex2f(+100,-100);
      glTexCoord2f(1,1); glVertex2f(+100,+100);
      glTexCoord2f(0,1); glVertex2f(-100,+100);
   glEnd;
   MyImage.Show(FALSE);

  TimeCount:= GetTickCount - StartTime;
  Inc(FrameCount);

  If TimeCount >= 1000 Then Begin
     FPS:= FrameCount;
     StartTime:= GetTickCount;
     TimeCount:= 0;
     FrameCount:= 0;
  End;

   MyFont.Show(AWindow.Width-120,10,'FPS: ' + IntToStr(FPS));

   //glDisable(GL_SCISSOR_TEST);
End;
//********************************************************************
Procedure RenderQuadOnIdle(AWindow: TGUIWindow);
Begin
   With AWindow Do Begin
      RenderContextActivate(Handle,LocalRenderContext);
      glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);
   End;

   RenderQuad(AWindow);
   AWindow.Show;

   With AWindow Do Begin
      WindowSwapBuffers(Handle);
      RenderContextDeactivate;
   End;
End;
//********************************************************************
Procedure RenderQuadOnPaint(AWindow: TGUIWindow);
Begin
   RenderQuad(AWindow);
End;
//********************************************************************

INITIALIZATION

Angle:=0;
FPS:= 0;
TimeCount:= 0;
FrameCount:= 0;
StartTime:= GetTickCount;

End.
